@@ -35,6 +35,7 @@
 #include <libanjuta/interfaces/ianjuta-editor-zoom.h>
 #include <libanjuta/interfaces/ianjuta-editor-goto.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
+#include <libanjuta/interfaces/ianjuta-editor-assist.h>
 
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -81,7 +82,7 @@ get_current_popup_active (gpointer user_data)
 	if (widget)
 	{
 		widget = gtk_widget_get_toplevel (widget);
-		if (GTK_WIDGET_TOPLEVEL (widget))
+		if (gtk_widget_is_toplevel (widget))
 			return gtk_window_has_toplevel_focus (GTK_WINDOW (widget));
 	}
 	return FALSE;
@@ -263,28 +264,36 @@ on_reload_file_activate (GtkAction *action, gpointer user_data)
 	gchar *msg;
 	GtkWidget *dialog;
 	GtkWidget *parent;
+	gint reply = GTK_RESPONSE_YES;
 	
 	doc = get_current_document (user_data);
 	if (doc == NULL)
 		return;
 
 	parent = gtk_widget_get_toplevel (GTK_WIDGET (doc));
-	msg = g_strdup_printf (
-	_("Are you sure you want to reload '%s'?\nAny unsaved changes will be lost."),
-							ianjuta_document_get_filename (doc, NULL));
+	if (IANJUTA_IS_FILE_SAVABLE (doc) && ianjuta_file_savable_is_dirty (IANJUTA_FILE_SAVABLE (doc), NULL))
+	{
+		msg = g_strdup_printf (
+		_("Are you sure you want to reload '%s'?\nAny unsaved changes will be lost."),
+								ianjuta_document_get_filename (doc, NULL));
 
-	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-									 GTK_DIALOG_DESTROY_WITH_PARENT,
-									 GTK_MESSAGE_QUESTION,
-									 GTK_BUTTONS_NONE, "%s", msg);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-						   GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
-	anjuta_util_dialog_add_button (GTK_DIALOG (dialog), _("_Reload"),
-							  GTK_STOCK_REVERT_TO_SAVED,
-							  GTK_RESPONSE_YES);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-									 GTK_RESPONSE_NO);
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+										 GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_MESSAGE_QUESTION,
+										 GTK_BUTTONS_NONE, "%s", msg);
+		gtk_dialog_add_button (GTK_DIALOG (dialog),
+							   GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
+		anjuta_util_dialog_add_button (GTK_DIALOG (dialog), _("_Reload"),
+								  GTK_STOCK_REVERT_TO_SAVED,
+								  GTK_RESPONSE_YES);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+										 GTK_RESPONSE_NO);
+		reply = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		g_free (msg);
+	}
+		
+	if (reply == GTK_RESPONSE_YES)
 	{
 		GFile* file;
 		file = ianjuta_file_get_file (IANJUTA_FILE (doc), NULL);
@@ -294,8 +303,6 @@ on_reload_file_activate (GtkAction *action, gpointer user_data)
 			g_object_unref(file);
 		}
 	}
-	gtk_widget_destroy (dialog);
-	g_free (msg);
 }
 
 void
@@ -421,16 +428,6 @@ on_editor_command_select_all_activate (GtkAction *action, gpointer user_data)
 	}
 }
 
-void
-on_editor_command_select_to_brace_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_editor_selection_select_to_brace (IANJUTA_EDITOR_SELECTION (doc), NULL);
-}
-
 void
 on_editor_command_select_block_activate (GtkAction *action, gpointer user_data)
 {
@@ -714,7 +711,7 @@ on_editor_linenos1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_LINENUMBERS_MARGIN, state);
 }
 
@@ -725,7 +722,7 @@ on_editor_markers1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_MARKER_MARGIN, state);
 }
 
@@ -736,7 +733,7 @@ on_editor_codefold1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_FOLD_MARGIN, state);
 }
 
@@ -747,7 +744,7 @@ on_editor_indentguides1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_INDENTATION_GUIDES, state);
 }
 
@@ -758,7 +755,7 @@ on_editor_whitespaces1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_WHITE_SPACES, state);
 }
 
@@ -769,7 +766,7 @@ on_editor_eolchars1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_EOL, state);
 }
 
@@ -780,7 +777,7 @@ on_editor_linewrap1_activate (GtkAction *action, gpointer user_data)
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	anjuta_preferences_set_int (plugin->prefs,
+	anjuta_preferences_set_bool (plugin->prefs,
 								VIEW_LINE_WRAP, state);
 }
 
@@ -848,7 +845,7 @@ on_repeat_quicksearch (GtkAction *action, gpointer user_data)
 	if (!gtk_widget_get_parent (search_box))
 		gtk_box_pack_end (GTK_BOX (plugin->vbox), search_box, FALSE, FALSE, 0);
 
-	if (!GTK_WIDGET_VISIBLE (search_box))
+	if (!gtk_widget_get_visible (search_box))
 		gtk_widget_show (search_box);
 	on_search_activated (NULL, SEARCH_BOX (search_box));
 }
@@ -960,3 +957,17 @@ on_bookmarks_clear_activate (GtkAction *action, gpointer user_data)
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	anjuta_bookmarks_clear (ANJUTA_BOOKMARKS(plugin->bookmarks));
 }
+
+void
+on_autocomplete_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR_ASSIST(doc))
+	{
+		IAnjutaEditorAssist* assist = IANJUTA_EDITOR_ASSIST (doc);
+		ianjuta_editor_assist_invoke (assist, NULL, NULL);
+	}   
+}
