@@ -23,7 +23,6 @@
  */
 
 #include <config.h>
-#include <glade/glade-xml.h>
 #include <gio/gio.h>
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
@@ -35,10 +34,10 @@
 #include <libanjuta/interfaces/ianjuta-preferences.h>
 #include "plugin.h"
 
-#define UI_FILE PACKAGE_DATA_DIR"/ui/file-manager.ui"
+#define UI_FILE PACKAGE_DATA_DIR"/ui/file-manager.xml"
 #define ICON_FILE "anjuta-file-manager-plugin-48.png"
-#define FILE_MANAGER_GLADE PACKAGE_DATA_DIR"/glade/file-manager.glade"
-#define FILE_MANAGER_GLADE_ROOT "filemanager_prefs"
+#define FILE_MANAGER_BUILDER PACKAGE_DATA_DIR"/glade/file-manager.ui"
+#define FILE_MANAGER_BUILDER_ROOT "filemanager_prefs"
 
 #define PREF_ROOT "filemanager.root"
 #define PREF_FILTER_BINARY "filemanager.filter.binary"
@@ -46,8 +45,8 @@
 #define PREF_FILTER_BACKUP "filemanager.filter.backup"
 #define PREF_FILTER_UNVERSIONED "filemanager.filter.unversioned"
 
-#define REGISTER_NOTIFY(key, func) \
-	notify_id = anjuta_preferences_notify_add (file_manager->prefs, \
+#define REGISTER_NOTIFY(key, func, type) \
+	notify_id = anjuta_preferences_notify_add_##type (file_manager->prefs, \
 											   key, func, file_manager, NULL); \
 	file_manager->gconf_notify_ids = g_list_prepend (file_manager->gconf_notify_ids, \
 										   GUINT_TO_POINTER(notify_id));
@@ -156,24 +155,28 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 					const GValue *value, gpointer user_data)
 {
 	AnjutaFileManager* file_manager;
+	GtkTreeModelSort *tree_model;
+	FileModel *file_model;
 	const gchar *root_uri;
 
 	file_manager = (AnjutaFileManager*) plugin;
+	tree_model = GTK_TREE_MODEL_SORT (
+	    gtk_tree_view_get_model (GTK_TREE_VIEW (file_manager->fv)));
+	file_model = FILE_MODEL (gtk_tree_model_sort_get_model (tree_model));
+	
 	root_uri = g_value_get_string (value);
 	if (root_uri)
 	{
 		g_object_set (G_OBJECT(file_manager->fv), "base_uri", root_uri, NULL);
-		g_object_set_data (G_OBJECT(file_manager->fv), "__ivcs",
-						   get_vcs_plugin (file_manager,
-										   root_uri));
+		file_model_set_ivcs (file_model, get_vcs_plugin (file_manager,
+														 root_uri));
 		
 		file_view_refresh (file_manager->fv);
 		file_manager->have_project = TRUE;
 	}
 	else
 	{
-		g_object_set_data (G_OBJECT(file_manager->fv), "__ivcs",
-						   NULL);
+		file_model_set_ivcs(file_model, NULL);
 		file_manager_set_default_uri (file_manager);
 		file_view_refresh (file_manager->fv);
 	}
@@ -243,8 +246,10 @@ on_file_view_show_popup_menu (AnjutaFileView* view, GFile* file,
 }
 
 static void 
-on_gconf_notify_root(GConfClient *gclient, guint cnxn_id,
-					 GConfEntry *entry, gpointer user_data)
+on_notify_root(AnjutaPreferences* prefs,
+                         const gchar* key,
+                         const gchar* value,
+                         gpointer user_data)
 {
 	AnjutaFileManager* file_manager = (AnjutaFileManager*) user_data;
 	if (!file_manager->have_project)
@@ -253,19 +258,22 @@ on_gconf_notify_root(GConfClient *gclient, guint cnxn_id,
 		file_view_refresh (file_manager->fv);
 	}
 }
+
 static void 
-on_gconf_notify(GConfClient *gclient, guint cnxn_id,
-				GConfEntry *entry, gpointer user_data)
+on_notify(AnjutaPreferences* prefs,
+          const gchar* key,
+          gboolean value,
+          gpointer user_data)
 {
 	AnjutaFileManager* file_manager = (AnjutaFileManager*) user_data;
 	GtkTreeModel* sort_model = gtk_tree_view_get_model (GTK_TREE_VIEW (file_manager->fv));
 	GtkTreeModel* file_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT(sort_model));
 	
 	g_object_set (G_OBJECT (file_model),
-				  "filter_binary", anjuta_preferences_get_int (file_manager->prefs, PREF_FILTER_BINARY),
-				  "filter_hidden", anjuta_preferences_get_int (file_manager->prefs, PREF_FILTER_HIDDEN),
-				  "filter_backup", anjuta_preferences_get_int (file_manager->prefs, PREF_FILTER_BACKUP),
-				  "filter_unversioned", anjuta_preferences_get_int (file_manager->prefs, PREF_FILTER_UNVERSIONED), NULL);
+				  "filter_binary", anjuta_preferences_get_bool (file_manager->prefs, PREF_FILTER_BINARY),
+				  "filter_hidden", anjuta_preferences_get_bool (file_manager->prefs, PREF_FILTER_HIDDEN),
+				  "filter_backup", anjuta_preferences_get_bool (file_manager->prefs, PREF_FILTER_BACKUP),
+				  "filter_unversioned", anjuta_preferences_get_bool (file_manager->prefs, PREF_FILTER_UNVERSIONED), NULL);
 	file_view_refresh (file_manager->fv);
 }
 
@@ -329,12 +337,12 @@ file_manager_activate (AnjutaPlugin *plugin)
 								 project_root_removed, NULL);
 	
 	
-	REGISTER_NOTIFY (PREF_ROOT, on_gconf_notify_root);
-	REGISTER_NOTIFY (PREF_FILTER_BINARY, on_gconf_notify);
-	REGISTER_NOTIFY (PREF_FILTER_BACKUP, on_gconf_notify);
-	REGISTER_NOTIFY (PREF_FILTER_HIDDEN, on_gconf_notify);
-	REGISTER_NOTIFY (PREF_FILTER_UNVERSIONED, on_gconf_notify);
-	on_gconf_notify (NULL, 0, NULL, file_manager);
+	REGISTER_NOTIFY (PREF_ROOT, on_notify_root, string);
+	REGISTER_NOTIFY (PREF_FILTER_BINARY, on_notify, bool);
+	REGISTER_NOTIFY (PREF_FILTER_BACKUP, on_notify, bool);
+	REGISTER_NOTIFY (PREF_FILTER_HIDDEN, on_notify, bool);
+	REGISTER_NOTIFY (PREF_FILTER_UNVERSIONED, on_notify, bool);
+	on_notify (anjuta_preferences_default(), NULL, FALSE, file_manager);
 	
 	return TRUE;
 }
@@ -440,11 +448,16 @@ ipreferences_merge (IAnjutaPreferences* ipref,
 					AnjutaPreferences* prefs,
 					GError** e)
 {
-	GladeXML* gxml;
-	
-	gxml = glade_xml_new (FILE_MANAGER_GLADE, FILE_MANAGER_GLADE_ROOT, NULL);
-	
-	anjuta_preferences_add_page (prefs, gxml, FILE_MANAGER_GLADE_ROOT, _("File Manager"),
+	GError* error = NULL;
+	GtkBuilder* bxml = gtk_builder_new ();
+
+	if (!gtk_builder_add_from_file (bxml, FILE_MANAGER_BUILDER, &error))
+	{
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
+	}
+
+	anjuta_preferences_add_from_builder (prefs, bxml, FILE_MANAGER_BUILDER_ROOT, _("File Manager"),
 								 ICON_FILE);
 }
 
