@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * anjuta
  * Copyright (C) Massimo Cora' 2007-2009 <maxcvs@email.it>
@@ -36,7 +37,7 @@ isymbol_manager_search (IAnjutaSymbolManager *sm,
 						IAnjutaSymbolField info_fields,
 						const gchar *match_name,
 						gboolean partial_name_match,
-						gboolean global_symbols_search,
+						IAnjutaSymbolManagerSearchFileScope filescope_search,
 						gboolean global_tags_search,
 						gint results_limit,
 						gint results_offset,
@@ -46,7 +47,6 @@ isymbol_manager_search (IAnjutaSymbolManager *sm,
 	SymbolDBPlugin *sdb_plugin;
 	SymbolDBEngine *dbe_project;
 	SymbolDBEngine *dbe_globals;
-	GPtrArray *filter_array;
 	gchar *pattern;
 	gboolean exact_match = !partial_name_match;
 
@@ -54,25 +54,11 @@ isymbol_manager_search (IAnjutaSymbolManager *sm,
 	dbe_project = SYMBOL_DB_ENGINE (sdb_plugin->sdbe_project);
 	dbe_globals = SYMBOL_DB_ENGINE (sdb_plugin->sdbe_globals);
 	
-	if (match_types & IANJUTA_SYMBOL_TYPE_UNDEF)
-	{
-		filter_array = NULL;
-		/*DEBUG_PRINT ("%s", "filter_array is NULL");*/
-	}
-	else 
-	{
-		filter_array = symbol_db_util_fill_type_array (match_types);
-		/*DEBUG_PRINT ("filter_array filled with %d kinds", filter_array->len);*/
-	}
-
 	if (exact_match == FALSE)
 		pattern = g_strdup_printf ("%s%%", match_name == NULL ? "" : match_name);
 	else
 	{
-		if (match_name == NULL)
-			pattern = NULL;
-		else
-			pattern = g_strdup_printf ("%s", match_name);
+		pattern = g_strdup_printf ("%s", match_name == NULL ? "%" : match_name);
 	}
 	
 	/* should we lookup for project of system tags? */
@@ -80,21 +66,15 @@ isymbol_manager_search (IAnjutaSymbolManager *sm,
 		symbol_db_engine_find_symbol_by_name_pattern_filtered (
 					global_tags_search == FALSE ? dbe_project : dbe_globals, 
 					pattern,
-					exact_match,
-					filter_array,
+					match_types,
 					include_types,
-					global_symbols_search,
+					filescope_search,
 					global_tags_search == FALSE ? NULL : sdb_plugin->session_packages,
 					results_limit,
 					results_offset,
 					info_fields);	
 	g_free (pattern);
 	
-	if (filter_array)
-	{
-		g_ptr_array_foreach (filter_array, (GFunc)g_free, NULL);
-		g_ptr_array_free (filter_array, TRUE);
-	}
 	return IANJUTA_ITERABLE (iterator);
 }
 
@@ -102,7 +82,6 @@ IAnjutaIterable*
 isymbol_manager_get_members (IAnjutaSymbolManager *sm,
 							 const IAnjutaSymbol *symbol, 
 							 IAnjutaSymbolField info_fields,
-							 gboolean global_search,
 							 GError **err)
 {
 	SymbolDBEngineIteratorNode *node;
@@ -201,6 +180,58 @@ isymbol_manager_get_parent_scope (IAnjutaSymbolManager *sm,
 	return IANJUTA_ITERABLE (iterator);
 }
 
+IAnjutaIterable*
+isymbol_manager_get_scope_chain (IAnjutaSymbolManager *sm,                                 
+                                 const gchar* filename, 
+                                 gulong line,
+                                 IAnjutaSymbolField info_fields,
+                                 GError **err)
+{
+	SymbolDBPlugin *sdb_plugin;
+	SymbolDBEngine *dbe;
+	SymbolDBEngineIterator *iterator;
+
+	sdb_plugin = ANJUTA_PLUGIN_SYMBOL_DB (sm);
+	dbe = SYMBOL_DB_ENGINE (sdb_plugin->sdbe_project);
+
+	iterator = symbol_db_engine_get_scope_chain_by_file_line (dbe, filename, 
+	                                                          line, info_fields);
+
+	return IANJUTA_ITERABLE (iterator);
+}
+ 
+IAnjutaIterable* 
+isymbol_manager_search_symbol_in_scope (IAnjutaSymbolManager *sm,
+                                        const gchar *pattern, 
+                                        const IAnjutaSymbol *container_symbol, 
+                                        IAnjutaSymbolType match_types, 
+                                        gboolean include_types, 
+                                        gint results_limit, 
+                                        gint results_offset, 
+                                        IAnjutaSymbolField info_fields,
+                                        GError **err)
+{
+	SymbolDBPlugin *sdb_plugin;
+	SymbolDBEngine *dbe;
+	SymbolDBEngineIterator *iterator;
+	SymbolDBEngineIteratorNode *node;
+	gint container_sym_id;
+
+	sdb_plugin = ANJUTA_PLUGIN_SYMBOL_DB (sm);
+	dbe = SYMBOL_DB_ENGINE (sdb_plugin->sdbe_project);
+	node = SYMBOL_DB_ENGINE_ITERATOR_NODE (container_symbol);
+
+	container_sym_id = symbol_db_engine_iterator_node_get_symbol_id (node);
+	
+	iterator = 	symbol_db_engine_find_symbol_in_scope (dbe, pattern, container_sym_id ,
+		                                                  match_types, include_types,
+		                                                  results_limit,
+		                                                  results_offset,
+		                                                  info_fields);		                                                  
+
+	return IANJUTA_ITERABLE (iterator);
+}
+
 IAnjutaIterable*
 isymbol_manager_get_symbol_more_info (IAnjutaSymbolManager *sm,
 								  const IAnjutaSymbol *symbol, 
@@ -256,31 +287,18 @@ isymbol_manager_get_symbol_by_id (IAnjutaSymbolManager *sm,
 static SymbolDBEngineIterator *
 do_search_prj_glb (SymbolDBEngine *dbe, IAnjutaSymbolType match_types,
            gboolean include_types, IAnjutaSymbolField info_fields,
-           const gchar *pattern, gint results_limit, gint results_offset, 
-           GList *session_packages)
+           const gchar *pattern, IAnjutaSymbolManagerSearchFileScope filescope_search,
+    	   gint results_limit, gint results_offset, 
+           const GList *session_packages)
 {
 	SymbolDBEngineIterator *iterator;
-	gboolean exact_match;
-	GPtrArray *filter_array;
-	
-	exact_match = symbol_db_util_is_pattern_exact_match (pattern);
-
-	if (match_types & IANJUTA_SYMBOL_TYPE_UNDEF)
-	{
-		filter_array = NULL;
-	}
-	else 
-	{
-		filter_array = symbol_db_util_fill_type_array (match_types);
-	}
 	
 	iterator = 		
 		symbol_db_engine_find_symbol_by_name_pattern_filtered (dbe,
 					pattern,
-					exact_match,
-					filter_array,
+					match_types,
 					include_types,
-					1,
+					filescope_search,
 					session_packages,
 					results_limit,
 					results_offset,
@@ -292,7 +310,8 @@ do_search_prj_glb (SymbolDBEngine *dbe, IAnjutaSymbolType match_types,
 IAnjutaIterable* 
 isymbol_manager_search_system (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_types, 
 				gboolean include_types,  IAnjutaSymbolField info_fields, 
-				const gchar *pattern, gint results_limit, gint results_offset, 
+				const gchar *pattern, IAnjutaSymbolManagerSearchFileScope filescope_search,
+    			gint results_limit, gint results_offset, 
 				GError **err)
 {
 	SymbolDBPlugin *sdb_plugin;
@@ -304,8 +323,8 @@ isymbol_manager_search_system (IAnjutaSymbolManager *sm, IAnjutaSymbolType match
 	/* take the global's engine */
 	dbe = SYMBOL_DB_ENGINE (sdb_plugin->sdbe_globals);
 
-	iterator = do_search_prj_glb (dbe, match_types, info_fields, 
-	                      include_types, pattern, 
+	iterator = do_search_prj_glb (dbe, match_types, include_types, 
+	                      info_fields, pattern, filescope_search,
 						  results_limit, results_offset,
 	           			  sdb_plugin->session_packages);
 
@@ -328,6 +347,15 @@ on_sdb_search_command_data_arrived (AnjutaCommand *command,
 	guint cmd_id;
 	
 	sdbsc = SYMBOL_DB_SEARCH_COMMAND (command);
+
+	/* If the command is canceled, don't bother with the callback.
+	 * Blocking the data-arrived signal in SymbolDBSearchCommand doesn't
+	 * seem to be enough because this signal can get emitted in idle
+	 * loop before cancel happened.
+	 */
+	if (symbol_db_search_command_get_cancelled (sdbsc))
+	  return;
+
 	iterator = symbol_db_search_command_get_iterator_result (sdbsc);
 		
 	/* get the previously saved cmd_id and callback data */
@@ -337,6 +365,45 @@ on_sdb_search_command_data_arrived (AnjutaCommand *command,
 	callback (cmd_id, IANJUTA_ITERABLE (iterator), callback_user_data);
 }
 
+static void
+on_sdb_search_command_cancelled (GCancellable *cancellable,
+								 AnjutaCommand *command)
+{
+	anjuta_command_cancel (command);
+
+	/* Since cancellable is external object, we need to clean our part
+	 * to not possibily get spurious callbacks when the cancellable is
+	 * reused in different context
+	 */
+	g_signal_handlers_disconnect_by_func (cancellable,
+										  on_sdb_search_command_cancelled,
+										  command);
+	g_object_set_data (G_OBJECT (command), "cancellable", NULL);
+	g_object_unref (cancellable);
+}
+
+static void
+on_sdb_search_command_finished (SymbolDBSearchCommand *search_command,
+								guint return_code,
+								AnjutaAsyncNotify *notify)
+{
+	GCancellable *cancel;
+
+	DEBUG_PRINT ("sdb search finished %p", search_command);
+
+	if (!symbol_db_search_command_get_cancelled (search_command) && notify)
+		anjuta_async_notify_notify_finished (notify);
+
+	cancel = g_object_get_data (G_OBJECT (search_command), "cancellable");
+	if (cancel)
+	{
+		g_signal_handlers_disconnect_by_func (cancel, on_sdb_search_command_cancelled, search_command);
+		g_object_unref (cancel);
+		g_object_set_data (G_OBJECT (search_command), "cancellable", NULL);
+	}
+	g_object_unref (search_command); /* Fully done */
+}
+
 static gint
 do_search_prj_glb_async (SymbolDBSearchCommand *search_command, guint cmd_id, 
                          GCancellable* cancel, AnjutaAsyncNotify *notify, 
@@ -362,21 +429,15 @@ do_search_prj_glb_async (SymbolDBSearchCommand *search_command, guint cmd_id,
 					  callback);
 	
 	g_signal_connect (G_OBJECT (search_command), "command-finished",
-					  G_CALLBACK (g_object_unref),
-					  NULL);
-	
+					  G_CALLBACK (on_sdb_search_command_finished),
+					  notify);
 	if (cancel)
 	{
-		g_signal_connect_swapped (G_OBJECT (cancel), "cancelled",
-								  G_CALLBACK (anjuta_command_cancel),
-								  search_command);
-	}
-	
-	if (notify)
-	{
-		g_signal_connect_swapped (G_OBJECT (search_command), "command-finished",
-								  G_CALLBACK (anjuta_async_notify_notify_finished),
-								  notify);
+		g_object_ref (cancel);
+		g_signal_connect (G_OBJECT (cancel), "cancelled",
+		    			  G_CALLBACK (on_sdb_search_command_cancelled),
+						  search_command);
+		g_object_set_data (G_OBJECT (search_command), "cancellable", cancel);
 	}
 	
 	anjuta_command_start (ANJUTA_COMMAND (search_command));	
@@ -386,7 +447,8 @@ do_search_prj_glb_async (SymbolDBSearchCommand *search_command, guint cmd_id,
 
 guint
 isymbol_manager_search_system_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_types, 
-				gboolean include_types,  IAnjutaSymbolField info_fields, const gchar *pattern, 
+				gboolean include_types,  IAnjutaSymbolField info_fields, const gchar *pattern,
+    			IAnjutaSymbolManagerSearchFileScope filescope_search,
 			    gint results_limit, gint results_offset, 
                 GCancellable* cancel, AnjutaAsyncNotify *notify, 
                 IAnjutaSymbolManagerSearchCallback callback, 
@@ -410,7 +472,8 @@ isymbol_manager_search_system_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType
 	
 	/* create a new command */
 	search_command = symbol_db_search_command_new (dbe, CMD_SEARCH_SYSTEM, match_types, 
-				include_types, info_fields, pattern, results_limit, results_offset);
+				include_types, info_fields, pattern, filescope_search, results_limit, 
+	    		results_offset);
 
 	/* don't forget to set the session packages too */
 	symbol_db_search_command_set_session_packages (search_command, 
@@ -423,6 +486,7 @@ isymbol_manager_search_system_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType
 IAnjutaIterable* 
 isymbol_manager_search_project (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_types, 
 				gboolean include_types,  IAnjutaSymbolField info_fields, const gchar *pattern, 
+    			IAnjutaSymbolManagerSearchFileScope filescope_search,
 				gint results_limit, gint results_offset, GError **err)
 {
 	SymbolDBPlugin *sdb_plugin;
@@ -434,8 +498,8 @@ isymbol_manager_search_project (IAnjutaSymbolManager *sm, IAnjutaSymbolType matc
 	/* take the project's engine */
 	dbe = SYMBOL_DB_ENGINE (sdb_plugin->sdbe_project);
 
-	iterator = do_search_prj_glb (dbe, match_types, info_fields, 
-	                      include_types, pattern, 
+	iterator = do_search_prj_glb (dbe, match_types, include_types, 
+	                      info_fields, pattern, filescope_search,
 						  results_limit, results_offset,
 	           			  NULL);
 
@@ -445,6 +509,7 @@ isymbol_manager_search_project (IAnjutaSymbolManager *sm, IAnjutaSymbolType matc
 guint
 isymbol_manager_search_project_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_types, 
 				gboolean include_types,  IAnjutaSymbolField info_fields, const gchar *pattern, 
+    			IAnjutaSymbolManagerSearchFileScope filescope_search,
 				gint results_limit, gint results_offset, 
                 GCancellable* cancel, AnjutaAsyncNotify *notify, 
                 IAnjutaSymbolManagerSearchCallback callback, 
@@ -468,7 +533,8 @@ isymbol_manager_search_project_async (IAnjutaSymbolManager *sm, IAnjutaSymbolTyp
 	
 	/* create a new command */
 	search_command = symbol_db_search_command_new (dbe, CMD_SEARCH_PROJECT, match_types, 
-				include_types, info_fields, pattern, results_limit, results_offset);
+				include_types, info_fields, pattern, filescope_search, results_limit, 
+	    		results_offset);
 
 	/* don't forget to set the session packages to NULL */
 	symbol_db_search_command_set_session_packages (search_command, NULL);	
@@ -485,7 +551,6 @@ isymbol_manager_search_file (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_t
 	SymbolDBPlugin *sdb_plugin;
 	SymbolDBEngine *dbe;
 	SymbolDBEngineIterator *iterator;
-	GPtrArray *filter_array;
 	gchar *abs_file_path;
 
 	g_return_val_if_fail (file != NULL, NULL);
@@ -501,21 +566,12 @@ isymbol_manager_search_file (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_t
 		g_warning ("isymbol_manager_search_file (): GFile has no absolute path");
 		return NULL;
 	}
-	
-	if (match_types & IANJUTA_SYMBOL_TYPE_UNDEF)
-	{
-		filter_array = NULL;
-	}
-	else 
-	{
-		filter_array = symbol_db_util_fill_type_array (match_types);
-	}
-	
+		
 	iterator = 		
 		symbol_db_engine_find_symbol_by_name_pattern_on_file (dbe,
 				    pattern,
 					abs_file_path,
-					filter_array,
+					match_types,
 					include_types,
 					results_limit,
 					results_offset,
@@ -526,10 +582,9 @@ isymbol_manager_search_file (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_t
 	return IANJUTA_ITERABLE (iterator);
 }
 
-
 guint
 isymbol_manager_search_file_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType match_types, 
-				gboolean include_types,  IAnjutaSymbolField info_fields, const gchar *pattern, 
+				gboolean include_types,  IAnjutaSymbolField info_fields, const gchar *pattern,     			
 			 	const GFile *file, gint results_limit, gint results_offset, 
                 GCancellable* cancel, AnjutaAsyncNotify *notify, 
                 IAnjutaSymbolManagerSearchCallback callback, 
@@ -554,7 +609,9 @@ isymbol_manager_search_file_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType m
 	
 	/* create a new command */
 	search_command = symbol_db_search_command_new (dbe, CMD_SEARCH_FILE, match_types, 
-				include_types, info_fields, pattern, results_limit, results_offset);
+				include_types, info_fields, pattern, IANJUTA_SYMBOL_MANAGER_SEARCH_FS_IGNORE, 
+	    		results_limit, 
+	    		results_offset);
 
 	/* don't forget to set the file too */
 	symbol_db_search_command_set_file (search_command, file);	
@@ -578,21 +635,15 @@ isymbol_manager_search_file_async (IAnjutaSymbolManager *sm, IAnjutaSymbolType m
 					  callback);
 	
 	g_signal_connect (G_OBJECT (search_command), "command-finished",
-					  G_CALLBACK (g_object_unref),
-					  NULL);
-	
+					  G_CALLBACK (on_sdb_search_command_finished),
+					  notify);
 	if (cancel)
 	{
-		g_signal_connect_swapped (G_OBJECT (cancel), "cancelled",
-								  G_CALLBACK (anjuta_command_cancel),
-								  search_command);
-	}
-	
-	if (notify)
-	{
-		g_signal_connect_swapped (G_OBJECT (search_command), "command-finished",
-								  G_CALLBACK (anjuta_async_notify_notify_finished),
-								  notify);
+		g_object_ref (cancel);
+		g_signal_connect (G_OBJECT (cancel), "cancelled",
+						  G_CALLBACK (on_sdb_search_command_cancelled),
+						  search_command);
+		g_object_set_data (G_OBJECT (search_command), "cancellable", cancel);
 	}
 	
 	anjuta_command_start (ANJUTA_COMMAND (search_command));	
@@ -608,8 +659,10 @@ isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 	iface->get_class_parents = isymbol_manager_get_class_parents;
 	iface->get_scope = isymbol_manager_get_scope;
 	iface->get_parent_scope = isymbol_manager_get_parent_scope;
+	iface->get_scope_chain = isymbol_manager_get_scope_chain;
 	iface->get_symbol_more_info = isymbol_manager_get_symbol_more_info;
 	iface->get_symbol_by_id = isymbol_manager_get_symbol_by_id;
+	iface->search_symbol_in_scope = isymbol_manager_search_symbol_in_scope;
 	iface->search_system = isymbol_manager_search_system;
 	iface->search_system_async = isymbol_manager_search_system_async;
 	iface->search_project = isymbol_manager_search_project;
@@ -617,3 +670,67 @@ isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 	iface->search_file = isymbol_manager_search_file;
 	iface->search_file_async = isymbol_manager_search_file_async;	
 }
+
+void
+on_isymbol_manager_prj_symbol_inserted (SymbolDBEngine *dbe,
+    									gint symbol_id,
+    									IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "prj-symbol-inserted", symbol_id);
+}
+
+void
+on_isymbol_manager_prj_symbol_removed (SymbolDBEngine *dbe,
+    									gint symbol_id,
+    									IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "prj-symbol-removed", symbol_id);
+}
+
+void
+on_isymbol_manager_prj_symbol_updated (SymbolDBEngine *dbe,
+    									gint symbol_id,
+    									IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "prj-symbol-updated", symbol_id);
+}
+
+void
+on_isymbol_manager_prj_scan_end (SymbolDBEngine *dbe,
+    								gint process_id,
+    								IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "prj-scan-end", process_id);
+}
+
+void
+on_isymbol_manager_sys_symbol_inserted (SymbolDBEngine *dbe,
+    									gint symbol_id,
+    									IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "sys-symbol-inserted", symbol_id);
+}
+
+void
+on_isymbol_manager_sys_symbol_removed (SymbolDBEngine *dbe,
+    									gint symbol_id,
+    									IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "sys-symbol-removed", symbol_id);
+}
+
+void
+on_isymbol_manager_sys_symbol_updated (SymbolDBEngine *dbe,
+    									gint symbol_id,
+    									IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "sys-symbol-updated", symbol_id);
+}
+
+void
+on_isymbol_manager_sys_scan_end (SymbolDBEngine *dbe,
+    								gint process_id,
+    								IAnjutaSymbolManager *sm)
+{
+	g_signal_emit_by_name (sm, "sys-scan-end", process_id);
+}
