@@ -375,9 +375,8 @@ add_new_waiting_for (SymbolDBView *dbv, gint parent_symbol_id,
 	{
 		/* no lists already set. Create one. */
 		GSList *slist;					
-		slist = g_slist_alloc ();			
 				
-		slist = g_slist_prepend (slist, wfs);
+		slist = g_slist_prepend ((GSList *)NULL, wfs);
 
 		/* add it to the binary tree. */
 		g_tree_insert (priv->waiting_for, GINT_TO_POINTER (parent_symbol_id), 
@@ -796,7 +795,7 @@ sdb_view_do_add_hidden_dummy_child (SymbolDBView *dbv, SymbolDBEngine *dbe,
 		gtk_tree_store_append (store, &child_iter, parent_iter);			
 		gtk_tree_store_set (store, &child_iter,
 					COLUMN_PIXBUF, NULL,
-					COLUMN_NAME, _("Loading..."),
+					COLUMN_NAME, _("Loading…"),
 					COLUMN_SYMBOL_ID, DUMMY_SYMBOL_ID,
 					-1);
 
@@ -1446,6 +1445,13 @@ sdb_view_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+sdb_view_dispose (GObject *object)
+{
+	/*DEBUG_PRINT ("sdb_view_dispose ");*/
+	G_OBJECT_CLASS (parent_class)->dispose (object);	
+}
+
 static void
 sdb_view_class_init (SymbolDBViewClass *klass)
 {
@@ -1455,6 +1461,7 @@ sdb_view_class_init (SymbolDBViewClass *klass)
 
 	sdbc = SYMBOL_DB_VIEW_CLASS (klass);
 	object_class->finalize = sdb_view_finalize;
+	object_class->dispose = sdb_view_dispose;
 }
 
 GType
